@@ -19,6 +19,17 @@
 
 #define MYSQL_DEFAULT_SQL_MODE	""
 #define MYSQL_DEFAULT_TIME_ZONE	"SYSTEM"
+#define MYSQL_DEFAULT_ISOLATION_LEVEL	"READ COMMITTED"
+#define MYSQL_DEFAULT_TRANSACTION_READ	"WRITE"
+#define MYSQL_DEFAULT_TX_ISOLATION	"READ-COMMITTED"
+#define MYSQL_DEFAULT_CHARACTER_SET_RESULTS	"NULL"
+#define MYSQL_DEFAULT_SESSION_TRACK_GTIDS	"OFF"
+#define MYSQL_DEFAULT_SQL_AUTO_IS_NULL	"OFF"
+#define MYSQL_DEFAULT_SQL_SELECT_LIMIT	"DEFAULT"
+#define MYSQL_DEFAULT_SQL_SAFE_UPDATES	"OFF"
+#define MYSQL_DEFAULT_COLLATION_CONNECTION	""
+#define MYSQL_DEFAULT_NET_WRITE_TIMEOUT	"60"
+#define MYSQL_DEFAULT_MAX_JOIN_SIZE	"18446744073709551615"
 
 static unsigned int near_pow_2 (unsigned int n) {
   unsigned int i = 1;
@@ -356,6 +367,7 @@ class MySQL_Threads_Handler
 		int monitor_replication_lag_timeout;
 		int monitor_groupreplication_healthcheck_interval;
 		int monitor_groupreplication_healthcheck_timeout;
+		int monitor_groupreplication_healthcheck_max_timeout_count;
 		int monitor_galera_healthcheck_interval;
 		int monitor_galera_healthcheck_timeout;
 		int monitor_galera_healthcheck_max_timeout_count;
@@ -397,6 +409,7 @@ class MySQL_Threads_Handler
 		bool query_digests;
 		bool query_digests_lowercase;
 		bool query_digests_replace_null;
+		bool query_digests_no_digits;
 		bool query_digests_normalize_digest_text;
 		bool query_digests_track_hostname;
 		bool default_reconnect;
@@ -411,6 +424,8 @@ class MySQL_Threads_Handler
 		bool autocommit_false_is_transaction;
 		bool verbose_query_error;
 		int max_allowed_packet;
+		bool use_tcp_keepalive;
+		int tcp_keepalive_time;
 		int throttle_connections_per_sec_to_hostgroup;
 		int max_transaction_time;
 		int threshold_query_length;
@@ -441,6 +456,17 @@ class MySQL_Threads_Handler
 		char *add_ldap_user_comment;
 		char *default_sql_mode;
 		char *default_time_zone;
+		char *default_isolation_level;
+		char *default_transaction_read;
+		char *default_tx_isolation;
+		char *default_character_set_results;
+		char *default_session_track_gtids;
+		char *default_sql_auto_is_null;
+		char *default_sql_select_limit;
+		char *default_sql_safe_updates;
+		char *default_collation_connection;
+		char *default_net_write_timeout;
+		char *default_max_join_size;
 #ifdef DEBUG
 		bool session_debug;
 #endif /* DEBUG */
