@@ -152,6 +152,17 @@ enum session_status {
 	SETTING_SQL_LOG_BIN,
 	SETTING_SQL_MODE,
 	SETTING_TIME_ZONE,
+	SETTING_ISOLATION_LEVEL,
+	SETTING_TRANSACTION_READ,
+	SETTING_TX_ISOLATION,
+	SETTING_CHARACTER_SET_RESULTS,
+	SETTING_SESSION_TRACK_GTIDS,
+	SETTING_SQL_AUTO_IS_NULL,
+	SETTING_SQL_SELECT_LIMIT,
+	SETTING_SQL_SAFE_UPDATES,
+	SETTING_COLLATION_CONNECTION,
+	SETTING_NET_WRITE_TIMEOUT,
+	SETTING_MAX_JOIN_SIZE,
 	FAST_FORWARD,
 	PROCESSING_STMT_PREPARE,
 	PROCESSING_STMT_EXECUTE,
@@ -612,7 +623,20 @@ __thread char *mysql_thread___init_connect;
 __thread char *mysql_thread___ldap_user_variable;
 __thread char *mysql_thread___default_sql_mode;
 __thread char *mysql_thread___default_time_zone;
+__thread char *mysql_thread___default_isolation_level;
+__thread char *mysql_thread___default_transaction_read;
+__thread char *mysql_thread___default_tx_isolation;
+__thread char *mysql_thread___default_character_set_results;
+__thread char *mysql_thread___default_session_track_gtids;
+__thread char *mysql_thread___default_sql_auto_is_null;
+__thread char *mysql_thread___default_sql_select_limit;
+__thread char *mysql_thread___default_sql_safe_updates;
+__thread char *mysql_thread___default_collation_connection;
+__thread char *mysql_thread___default_net_write_timeout;
+__thread char *mysql_thread___default_max_join_size;
 __thread int mysql_thread___max_allowed_packet;
+__thread bool mysql_thread___use_tcp_keepalive;
+__thread int mysql_thread___tcp_keepalive_time;
 __thread int mysql_thread___throttle_connections_per_sec_to_hostgroup;
 __thread int mysql_thread___max_transaction_time;
 __thread int mysql_thread___threshold_query_length;
@@ -665,6 +689,7 @@ __thread bool mysql_thread___commands_stats;
 __thread bool mysql_thread___query_digests;
 __thread bool mysql_thread___query_digests_lowercase;
 __thread bool mysql_thread___query_digests_replace_null;
+__thread bool mysql_thread___query_digests_no_digits;
 __thread bool mysql_thread___query_digests_normalize_digest_text;
 __thread bool mysql_thread___query_digests_track_hostname;
 __thread int mysql_thread___query_digests_max_digest_length;
@@ -714,6 +739,7 @@ __thread int mysql_thread___monitor_replication_lag_interval;
 __thread int mysql_thread___monitor_replication_lag_timeout;
 __thread int mysql_thread___monitor_groupreplication_healthcheck_interval;
 __thread int mysql_thread___monitor_groupreplication_healthcheck_timeout;
+__thread int mysql_thread___monitor_groupreplication_healthcheck_max_timeout_count;
 __thread int mysql_thread___monitor_galera_healthcheck_interval;
 __thread int mysql_thread___monitor_galera_healthcheck_timeout;
 __thread int mysql_thread___monitor_galera_healthcheck_max_timeout_count;
@@ -746,7 +772,20 @@ extern __thread char *mysql_thread___init_connect;
 extern __thread char *mysql_thread___ldap_user_variable;
 extern __thread char *mysql_thread___default_sql_mode;
 extern __thread char *mysql_thread___default_time_zone;
+extern __thread char *mysql_thread___default_isolation_level;
+extern __thread char *mysql_thread___default_transaction_read;
+extern __thread char *mysql_thread___default_tx_isolation;
+extern __thread char *mysql_thread___default_character_set_results;
+extern __thread char *mysql_thread___default_session_track_gtids;
+extern __thread char *mysql_thread___default_sql_auto_is_null;
+extern __thread char *mysql_thread___default_sql_select_limit;
+extern __thread char *mysql_thread___default_sql_safe_updates;
+extern __thread char *mysql_thread___default_collation_connection;
+extern __thread char *mysql_thread___default_net_write_timeout;
+extern __thread char *mysql_thread___default_max_join_size;
 extern __thread int mysql_thread___max_allowed_packet;
+extern __thread bool mysql_thread___use_tcp_keepalive;
+extern __thread int mysql_thread___tcp_keepalive_time;
 extern __thread int mysql_thread___throttle_connections_per_sec_to_hostgroup;
 extern __thread int mysql_thread___max_transaction_time;
 extern __thread int mysql_thread___threshold_query_length;
@@ -798,6 +837,7 @@ extern __thread bool mysql_thread___servers_stats;
 extern __thread bool mysql_thread___commands_stats;
 extern __thread bool mysql_thread___query_digests;
 extern __thread bool mysql_thread___query_digests_lowercase;
+extern __thread bool mysql_thread___query_digests_no_digits;
 extern __thread bool mysql_thread___query_digests_replace_null;
 extern __thread bool mysql_thread___query_digests_normalize_digest_text;
 extern __thread bool mysql_thread___query_digests_track_hostname;
@@ -848,6 +888,7 @@ extern __thread int mysql_thread___monitor_replication_lag_interval;
 extern __thread int mysql_thread___monitor_replication_lag_timeout;
 extern __thread int mysql_thread___monitor_groupreplication_healthcheck_interval;
 extern __thread int mysql_thread___monitor_groupreplication_healthcheck_timeout;
+extern __thread int mysql_thread___monitor_groupreplication_healthcheck_max_timeout_count;
 extern __thread int mysql_thread___monitor_galera_healthcheck_interval;
 extern __thread int mysql_thread___monitor_galera_healthcheck_timeout;
 extern __thread int mysql_thread___monitor_galera_healthcheck_max_timeout_count;
