@@ -28,9 +28,9 @@ static char * session_vars[]= {
 	// For issue #555 , multiplexing is disabled if --safe-updates is used
 	//(char *)"SQL_SAFE_UPDATES=?,SQL_SELECT_LIMIT=?,MAX_JOIN_SIZE=?",
 	// for issue #1832 , we are splitting the above into 3 variables
-	(char *)"SQL_SAFE_UPDATES",
-	(char *)"SQL_SELECT_LIMIT",
-	(char *)"MAX_JOIN_SIZE",
+//	(char *)"SQL_SAFE_UPDATES",
+//	(char *)"SQL_SELECT_LIMIT",
+//	(char *)"MAX_JOIN_SIZE",
 	(char *)"FOREIGN_KEY_CHECKS",
 	(char *)"UNIQUE_CHECKS",
 	(char *)"AUTO_INCREMENT_INCREMENT",
@@ -206,6 +206,28 @@ MySQL_Connection::MySQL_Connection() {
 	options.no_backslash_escapes=false;
 	options.init_connect=NULL;
 	options.init_connect_sent=false;
+	options.character_set_results = NULL;
+	options.isolation_level = NULL;
+	options.tx_isolation = NULL;
+	options.transaction_read = NULL;
+	options.session_track_gtids = NULL;
+	options.sql_auto_is_null = NULL;
+	options.sql_select_limit = NULL;
+	options.sql_safe_updates = NULL;
+	options.collation_connection = NULL;
+	options.net_write_timeout = NULL;
+	options.max_join_size = NULL;
+	options.isolation_level_sent = false;
+	options.tx_isolation_sent = false;
+	options.transaction_read_sent = false;
+	options.character_set_results_sent = false;
+	options.session_track_gtids_sent = false;
+	options.sql_auto_is_null_sent = false;
+	options.sql_select_limit_sent = false;
+	options.sql_safe_updates_sent = false;
+	options.collation_connection_sent = false;
+	options.net_write_timeout_sent = false;
+	options.max_join_size_sent = false;
 	options.sql_mode_sent=false;
 	options.ldap_user_variable=NULL;
 	options.ldap_user_variable_value=NULL;
@@ -215,6 +237,17 @@ MySQL_Connection::MySQL_Connection() {
 	options.sql_mode_int=0;	// #509
 	options.time_zone=NULL;	// #819
 	options.time_zone_int=0;	// #819
+	options.isolation_level_int=0;
+	options.tx_isolation_int=0;
+	options.transaction_read_int=0;
+	options.character_set_results_int=0;
+	options.session_track_gtids_int=0;
+	options.sql_auto_is_null_int=0;
+	options.sql_select_limit_int=0;
+	options.sql_safe_updates_int=0;
+	options.collation_connection_int=0;
+	options.net_write_timeout_int=0;
+	options.max_join_size_int=0;
 	compression_pkt_id=0;
 	mysql_result=NULL;
 	query.ptr=NULL;
@@ -237,7 +270,7 @@ MySQL_Connection::MySQL_Connection() {
 	statuses.questions = 0;
 	statuses.myconnpoll_get = 0;
 	statuses.myconnpoll_put = 0;
-
+	memset(gtid_uuid,0,sizeof(gtid_uuid));
 };
 
 MySQL_Connection::~MySQL_Connection() {
@@ -291,6 +324,50 @@ MySQL_Connection::~MySQL_Connection() {
 		free(options.time_zone);
 		options.time_zone=NULL;
 	}
+	if (options.isolation_level) {
+		free(options.isolation_level);
+		options.isolation_level=NULL;
+	}
+	if (options.tx_isolation) {
+		free(options.tx_isolation);
+		options.tx_isolation=NULL;
+	}
+	if (options.transaction_read) {
+		free(options.transaction_read);
+		options.transaction_read=NULL;
+	}
+	if (options.character_set_results) {
+		free(options.character_set_results);
+		options.character_set_results=NULL;
+	}
+	if (options.session_track_gtids) {
+		free(options.session_track_gtids);
+		options.session_track_gtids=NULL;
+	}
+	if (options.sql_auto_is_null) {
+		free(options.sql_auto_is_null);
+		options.sql_auto_is_null=NULL;
+	}
+	if (options.sql_select_limit) {
+		free(options.sql_select_limit);
+		options.sql_select_limit=NULL;
+	}
+	if (options.sql_safe_updates) {
+		free(options.sql_safe_updates);
+		options.sql_safe_updates=NULL;
+	}
+	if (options.collation_connection) {
+		free(options.collation_connection);
+		options.collation_connection=NULL;
+	}
+	if (options.net_write_timeout) {
+		free(options.net_write_timeout);
+		options.net_write_timeout=NULL;
+	}
+	if (options.max_join_size) {
+		free(options.max_join_size);
+		options.max_join_size=NULL;
+	}
 };
 
 bool MySQL_Connection::set_autocommit(bool _ac) {
@@ -1830,21 +1907,43 @@ void MySQL_Connection::ProcessQueryAndSetStatusFlags(char *query_digest_text) {
 		}
 	}
 	if (get_status_user_variable()==false) { // we search for variables only if not already set
-		if (mul!=2 && index(query_digest_text,'@')) { // mul = 2 has a special meaning : do not disable multiplex for variables in THIS QUERY ONLY
 //			if (
 //				strncasecmp(query_digest_text,"SELECT @@tx_isolation", strlen("SELECT @@tx_isolation"))
 //				&&
 //				strncasecmp(query_digest_text,"SELECT @@version", strlen("SELECT @@version"))
-			if (!IsKeepMultiplexEnabledVariables(query_digest_text)) {
-				set_status_user_variable(true);
-			}
-		}
 		if (strncasecmp(query_digest_text,"SET ",4)==0) {
 			// For issue #555 , multiplexing is disabled if --safe-updates is used (see session_vars definition)
-			for (unsigned int i = 0; i < sizeof(session_vars)/sizeof(char *); i++) {
-				if (strcasestr(query_digest_text,session_vars[i])!=NULL)  {
-					set_status_user_variable(true);
+			int sqloh = mysql_thread___set_query_lock_on_hostgroup;
+			switch (sqloh) {
+				case 0: // old algorithm
+					if (mul!=2) {
+						if (index(query_digest_text,'@')) { // mul = 2 has a special meaning : do not disable multiplex for variables in THIS QUERY ONLY
+							if (!IsKeepMultiplexEnabledVariables(query_digest_text)) {
+								set_status_user_variable(true);
+							}
+						} else {
+							for (unsigned int i = 0; i < sizeof(session_vars)/sizeof(char *); i++) {
+								if (strcasestr(query_digest_text,session_vars[i])!=NULL)  {
+									set_status_user_variable(true);
+									break;
+								}
+							}
+						}
+					}
 					break;
+				case 1: // new algorithm
+					if (myds->sess->locked_on_hostgroup > -1) {
+						// locked_on_hostgroup was set, so some variable wasn't parsed
+						set_status_user_variable(true);
+					}
+					break;
+				default:
+					break;
+			}
+		} else {
+			if (mul!=2 && index(query_digest_text,'@')) { // mul = 2 has a special meaning : do not disable multiplex for variables in THIS QUERY ONLY
+				if (!IsKeepMultiplexEnabledVariables(query_digest_text)) {
+					set_status_user_variable(true);
 				}
 			}
 		}
@@ -1997,6 +2096,72 @@ void MySQL_Connection::reset() {
 	delete local_stmts;
 	local_stmts=new MySQL_STMTs_local_v14(false);
 	creation_time = monotonic_time();
+	options.isolation_level_int = 0;
+	if (options.isolation_level) {
+		free (options.isolation_level);
+		options.isolation_level = NULL;
+		options.isolation_level_sent = false;
+	}
+	options.tx_isolation_int = 0;
+	if (options.tx_isolation) {
+		free (options.tx_isolation);
+		options.tx_isolation = NULL;
+		options.tx_isolation_sent = false;
+	}
+	options.transaction_read_int = 0;
+	if (options.transaction_read) {
+		free (options.transaction_read);
+		options.transaction_read = NULL;
+		options.transaction_read_sent = false;
+	}
+	options.character_set_results_int = 0;
+	if (options.character_set_results) {
+		free (options.character_set_results);
+		options.character_set_results = NULL;
+		options.character_set_results_sent = false;
+	}
+	options.session_track_gtids_int = 0;
+	if (options.session_track_gtids) {
+		free (options.session_track_gtids);
+		options.session_track_gtids = NULL;
+		options.session_track_gtids_sent = false;
+	}
+	options.sql_auto_is_null_int = 0;
+	if (options.sql_auto_is_null) {
+		free (options.sql_auto_is_null);
+		options.sql_auto_is_null = NULL;
+		options.sql_auto_is_null_sent = false;
+	}
+	options.sql_select_limit_int = 0;
+	if (options.sql_select_limit) {
+		free (options.sql_select_limit);
+		options.sql_select_limit = NULL;
+		options.sql_select_limit_sent = false;
+	}
+	options.sql_safe_updates_int = 0;
+	if (options.sql_safe_updates) {
+		free (options.sql_safe_updates);
+		options.sql_safe_updates = NULL;
+		options.sql_safe_updates_sent = false;
+	}
+	options.collation_connection_int = 0;
+	if (options.collation_connection) {
+		free (options.collation_connection);
+		options.collation_connection = NULL;
+		options.collation_connection_sent = false;
+	}
+	options.net_write_timeout_int = 0;
+	if (options.net_write_timeout) {
+		free (options.net_write_timeout);
+		options.net_write_timeout = NULL;
+		options.net_write_timeout_sent = false;
+	}
+	options.max_join_size_int = 0;
+	if (options.max_join_size) {
+		free (options.max_join_size);
+		options.max_join_size = NULL;
+		options.max_join_size_sent = false;
+	}
 	if (options.init_connect) {
 		free(options.init_connect);
 		options.init_connect = NULL;
@@ -2020,15 +2185,22 @@ bool MySQL_Connection::get_gtid(char *buff, uint64_t *trx_id) {
 		return ret;
 	}
 	if (mysql) {
-		const char *data;
-		size_t length;
-		if (mysql_session_track_get_first(mysql, SESSION_TRACK_GTIDS, &data, &length) == 0) {
-			if (memcmp(buff,data,length)) {
-				memcpy(buff,data,length);
-				buff[length]=0;
-				//fprintf(stderr,"GTID=%s\n",buff);
-				__sync_fetch_and_add(&myds->sess->thread->status_variables.gtid_session_collected,1);
-				ret = true;
+		if (mysql->net.last_errno==0) { // only if there is no error
+			if (mysql->server_status & SERVER_SESSION_STATE_CHANGED) { // only if status changed
+				const char *data;
+				size_t length;
+				if (mysql_session_track_get_first(mysql, SESSION_TRACK_GTIDS, &data, &length) == 0) {
+					if (memcmp(gtid_uuid,data,length)) {
+						// copy to local buffer in MySQL_Connection
+						memcpy(gtid_uuid,data,length);
+						gtid_uuid[length]=0;
+						// copy to external buffer in MySQL_Backend
+						memcpy(buff,data,length);
+						buff[length]=0;
+						__sync_fetch_and_add(&myds->sess->thread->status_variables.gtid_session_collected,1);
+						ret = true;
+					}
+				}
 			}
 		}
 	}
